@@ -161,9 +161,11 @@ do { \
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
 #define PMSG_EVENT(pmsg_state) (pmsg_state).event
-#define PM_EVENT_SUSPEND 2
 #else
 #define PMSG_EVENT(pmsg_state) (pmsg_state)
+/* For old kernels without PM_EVENT_xxx defines, define them 
+ * in consistent with the power state used in these kernels.
+ * */
 #define PM_EVENT_SUSPEND 3
 #endif
 
@@ -308,7 +310,6 @@ typedef struct {
 extern void          ATI_API_CALL KCL_GlobalKernelScheduler(void);
 extern int           ATI_API_CALL KCL_GetSignalStatus(void);
 extern unsigned int  ATI_API_CALL KCL_CurrentProcessIsTerminating(void);
-extern unsigned int  ATI_API_CALL KCL_ProcessIsTerminated(unsigned int pid);
 extern void          ATI_API_CALL KCL_SetCurrentProcessState(KCL_ENUM_ProcessState state);
 extern const char*   ATI_API_CALL KCL_GetModuleParamString(void);
 extern KCL_TYPE_Pid  ATI_API_CALL KCL_GetPid(void);
@@ -316,10 +317,11 @@ extern KCL_TYPE_Tgid ATI_API_CALL KCL_GetTgid(void);
 extern KCL_TYPE_Uid  ATI_API_CALL KCL_GetEffectiveUid(void);
 extern void          ATI_API_CALL KCL_DelayInMicroSeconds(unsigned long usecs);
 extern void          ATI_API_CALL KCL_DelayUseTSC(unsigned long usecs);
-extern unsigned long ATI_API_CALL KCL_ConvertAddressVirtualToBus(void* address);
 extern unsigned long ATI_API_CALL KCL_ConvertAddressVirtualToPhysical(void* address);
-
-extern unsigned long ATI_API_CALL KCL_ConvertPageToPfn(void* page);
+extern unsigned long long ATI_API_CALL KCL_MapVirtualToPhysical(KCL_PCI_DevHandle pdev, void* address, unsigned long size);
+extern void          ATI_API_CALL KCL_UnmapVirtualToPhysical(KCL_PCI_DevHandle pdev, unsigned long long bus_addr, unsigned long size);
+extern unsigned long ATI_API_CALL KCL_MapPageToPfn(KCL_PCI_DevHandle pdev, void* page);
+extern void          ATI_API_CALL KCL_UnmapPageToPfn(KCL_PCI_DevHandle pdev, unsigned long long bus_addr);
 extern void*         ATI_API_CALL KCL_ConvertPageToKernelAddress(void* page);
 extern unsigned int  ATI_API_CALL KCL_IsPageInHighMem(void* page);
 
@@ -328,7 +330,7 @@ extern int           ATI_API_CALL KCL_GetErrorCode(KCL_ENUM_ErrorCode errcode);
 extern void          ATI_API_CALL KCL_GlobalKernelLock(void);
 extern void          ATI_API_CALL KCL_GlobalKernelUnlock(void);
 extern int           ATI_API_CALL KCL_PosixSecurityCapCheck(KCL_ENUM_PosixSecurityCap cap);
-extern void          ATI_API_CALL KCL_PosixSecurityCapSetIPCLock(unsigned int lock);
+extern int           ATI_API_CALL KCL_PosixSecurityCapSetIPCLock(unsigned int lock);
 extern unsigned long ATI_API_CALL KCL_GetAvailableRamPages(void);
 extern void          ATI_API_CALL KCL_GetSystemMemInfo(KCL_SYS_MEM_INFO* info);
 extern void          ATI_API_CALL KCL_ReserveMemPage(void* pt);
@@ -479,6 +481,7 @@ extern void ATI_API_CALL KCL_MEM_FreePageForGart(void* pt);
 
 extern void ATI_API_CALL KCL_MEM_IncPageUseCount(void* pt);
 extern void ATI_API_CALL KCL_MEM_DecPageUseCount(void* pt);
+extern void ATI_API_CALL KCL_MEM_IncPageCount_Mapping(void* page);
 extern int ATI_API_CALL KCL_MEM_VerifyReadAccess(void* addr, __ke_size_t size);
 extern int ATI_API_CALL KCL_MEM_VerifyWriteAccess(void* addr, __ke_size_t size);
 extern unsigned long ATI_API_CALL KCL_GetPageTableByVirtAddr(unsigned long virtual_addr, unsigned long* page_addr);
@@ -509,12 +512,16 @@ extern int ATI_API_CALL KCL_is_pat_enabled(void);
 
 /*****************************************************************************/
 
-extern int ATI_API_CALL KCL_InstallInterruptHandler(unsigned int irq, void (*ATI_API_CALL handler)(void*), const char *dev_name, void *context);
+extern int ATI_API_CALL KCL_InstallInterruptHandler(unsigned int irq, void (*ATI_API_CALL handler)(void*), const char *dev_name, void *context, int useMSI);
 extern void ATI_API_CALL KCL_UninstallInterruptHandler(unsigned int irq, void *context);
 
+extern int ATI_API_CALL KCL_RequestMSI(void* context);
+extern void ATI_API_CALL KCL_DisableMSI(void* context);
+
 /*****************************************************************************/
 
 extern void* ATI_API_CALL KCL_MEM_VM_GetRegionFilePrivateData(struct vm_area_struct* vma);
+extern void* ATI_API_CALL KCL_MEM_VM_GetRegionPrivateData(struct vm_area_struct* vma);
 extern unsigned long ATI_API_CALL KCL_MEM_VM_GetRegionStart(struct vm_area_struct* vma);
 extern unsigned long ATI_API_CALL KCL_MEM_VM_GetRegionEnd(struct vm_area_struct* vma);
 extern unsigned long ATI_API_CALL KCL_MEM_VM_GetRegionMapOffset(struct vm_area_struct* vma);
@@ -542,7 +549,8 @@ extern int ATI_API_CALL KCL_MEM_VM_MapRegion(KCL_IO_FILE_Handle filp,
                                     struct vm_area_struct* vma,
                                     unsigned long long offset,
                                     enum __ke_vm_maptype type,
-                                    int readonly);
+                                    int readonly,
+                                    void *private_data);
 
 /*****************************************************************************/
 
@@ -561,6 +569,10 @@ extern void ATI_API_CALL firegl_pm_lock_highmem_gart(struct drm_device* dev, int
 
 /*****************************************************************************/
 
+extern int ATI_API_CALL KCL_PM_Is_SuspendToRam(unsigned int state);
+
+/*****************************************************************************/
+
 /* global constants */
 extern const char*          KCL_SYSINFO_OsVersionString;
 extern const unsigned long  KCL_SYSINFO_OsVersionCode;
@@ -593,6 +605,13 @@ extern unsigned long        KCL_SYSINFO_TimerTicksPerSecond;
 #define cpu_has_pge test_bit(X86_FEATURE_PGE, &boot_cpu_data.x86_capability)
 #endif
 
+/* 2.6.29 defines pgprot_writecombine as a macro which resolves to a
+ * GPL-only function with the same name. So we always use our own
+ * definition on 2.6.29 and later. */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29) \
+    && defined (pgprot_writecombine)
+#undef pgprot_writecombine
+#endif
 #ifndef pgprot_writecombine
 #define pgprot_writecombine(prot) __pgprot((pgprot_val(prot) & ~(_PAGE_PCD)) | _PAGE_PWT)
 #endif
